@@ -37,6 +37,8 @@ from werkzeug._compat import iterlistvalues
 from werkzeug._compat import itervalues
 from werkzeug._compat import PY2
 from werkzeug._compat import text_type
+from werkzeug.datastructures import LanguageAccept
+from werkzeug.datastructures import MIMEAccept
 from werkzeug.datastructures import Range
 from werkzeug.exceptions import BadRequestKeyError
 
@@ -583,11 +585,14 @@ class TestOrderedMultiDict(_MutableMultiDictTests):
         with pytest.raises(BadRequestKeyError) as exc_info:
             data["baz"]
 
+        assert "baz" not in exc_info.value.get_description()
+        exc_info.value.show_exception = True
         assert "baz" in exc_info.value.get_description()
 
         with pytest.raises(BadRequestKeyError) as exc_info:
             data.pop("baz")
 
+        exc_info.value.show_exception = True
         assert "baz" in exc_info.value.get_description()
         exc_info.value.args = ()
         assert "baz" not in exc_info.value.get_description()
@@ -775,6 +780,49 @@ class TestHeaders(object):
         assert h.get("x-whoops", as_bytes=True) == b"\xff"
         assert h.get("x-bytes") == "something"
 
+    def test_extend(self):
+        h = self.storage_class([("a", "0"), ("b", "1"), ("c", "2")])
+        h.extend(datastructures.Headers([("a", "3"), ("a", "4")]))
+        assert h.getlist("a") == ["0", "3", "4"]
+        h.extend(b=["5", "6"])
+        assert h.getlist("b") == ["1", "5", "6"]
+        h.extend({"c": "7", "d": ["8", "9"]}, c="10")
+        assert h.getlist("c") == ["2", "7", "10"]
+        assert h.getlist("d") == ["8", "9"]
+
+        with pytest.raises(TypeError):
+            h.extend({"x": "x"}, {"x": "x"})
+
+    def test_update(self):
+        h = self.storage_class([("a", "0"), ("b", "1"), ("c", "2")])
+        h.update(datastructures.Headers([("a", "3"), ("a", "4")]))
+        assert h.getlist("a") == ["3", "4"]
+        h.update(b=["5", "6"])
+        assert h.getlist("b") == ["5", "6"]
+        h.update({"c": "7", "d": ["8", "9"]})
+        assert h.getlist("c") == ["7"]
+        assert h.getlist("d") == ["8", "9"]
+        h.update({"c": "10"}, c="11")
+        assert h.getlist("c") == ["11"]
+
+        with pytest.raises(TypeError):
+            h.extend({"x": "x"}, {"x": "x"})
+
+    def test_setlist(self):
+        h = self.storage_class([("a", "0"), ("b", "1"), ("c", "2")])
+        h.setlist("b", ["3", "4"])
+        assert h[1] == ("b", "3")
+        assert h[-1] == ("b", "4")
+        h.setlist("b", [])
+        assert "b" not in h
+        h.setlist("d", ["5"])
+        assert h["d"] == "5"
+
+    def test_setlistdefault(self):
+        h = self.storage_class([("a", "0"), ("b", "1"), ("c", "2")])
+        assert h.setlistdefault("a", ["3"]) == ["0"]
+        assert h.setlistdefault("d", ["4", "5"]) == ["4", "5"]
+
     def test_to_wsgi_list(self):
         h = self.storage_class()
         h.set(u"Key", u"Value")
@@ -797,6 +845,20 @@ class TestHeaders(object):
                 strict_eq(key, u"Key")
                 strict_eq(value, u"Value")
 
+    def test_equality(self):
+        # test equality, given keys are case insensitive
+        h1 = self.storage_class()
+        h1.add("X-Foo", "foo")
+        h1.add("X-Bar", "bah")
+        h1.add("X-Bar", "humbug")
+
+        h2 = self.storage_class()
+        h2.add("x-foo", "foo")
+        h2.add("x-bar", "bah")
+        h2.add("x-bar", "humbug")
+
+        assert h1 == h2
+
 
 class TestEnvironHeaders(object):
     storage_class = datastructures.EnvironHeaders
@@ -894,12 +956,12 @@ def make_call_asserter(func=None):
 
     :param func: Additional callback for each function call.
 
-    >>> assert_calls, func = make_call_asserter()
-    >>> with assert_calls(2):
-    ...    func()
-    ...    func()
+    .. code-block:: python
+        assert_calls, func = make_call_asserter()
+        with assert_calls(2):
+            func()
+            func()
     """
-
     calls = [0]
 
     @contextmanager
@@ -961,6 +1023,32 @@ class TestCacheControl(object):
         cc = datastructures.RequestCacheControl([("max-age", "0"), ("private", "True")])
         assert repr(cc) == "<RequestCacheControl max-age='0' private='True'>"
 
+    def test_set_none(self):
+        cc = datastructures.ResponseCacheControl([("max-age", "0")])
+        assert cc.no_cache is None
+        cc.no_cache = None
+        assert cc.no_cache is None
+
+
+class TestContentSecurityPolicy(object):
+    def test_construct(self):
+        csp = datastructures.ContentSecurityPolicy(
+            [("font-src", "'self'"), ("media-src", "*")]
+        )
+        assert csp.font_src == "'self'"
+        assert csp.media_src == "*"
+        policies = [policy.strip() for policy in csp.to_header().split(";")]
+        assert "font-src 'self'" in policies
+        assert "media-src *" in policies
+
+    def test_properties(self):
+        csp = datastructures.ContentSecurityPolicy()
+        csp.default_src = "* 'self' quart.com"
+        csp.img_src = "'none'"
+        policies = [policy.strip() for policy in csp.to_header().split(";")]
+        assert "default-src * 'self' quart.com" in policies
+        assert "img-src 'none'" in policies
+
 
 class TestAccept(object):
     storage_class = datastructures.Accept
@@ -1038,23 +1126,78 @@ class TestAccept(object):
         assert accept.best_match(["asterisk", "times"], default=None) == "times"
         assert accept.best_match(["asterisk"], default=None) is None
 
+    def test_accept_equal_quality(self):
+        accept = self.storage_class([("a", 1), ("b", 1)])
+        assert accept.best == "a"
+
 
 class TestMIMEAccept(object):
-    storage_class = datastructures.MIMEAccept
+    @pytest.mark.parametrize(
+        ("values", "matches", "default", "expect"),
+        [
+            ([("text/*", 1)], ["text/html"], None, "text/html"),
+            ([("text/*", 1)], ["image/png"], "text/plain", "text/plain"),
+            ([("text/*", 1)], ["image/png"], None, None),
+            (
+                [("*/*", 1), ("text/html", 1)],
+                ["image/png", "text/html"],
+                None,
+                "text/html",
+            ),
+            (
+                [("*/*", 1), ("text/html", 1)],
+                ["image/png", "text/plain"],
+                None,
+                "image/png",
+            ),
+            (
+                [("*/*", 1), ("text/html", 1), ("image/*", 1)],
+                ["image/png", "text/html"],
+                None,
+                "text/html",
+            ),
+            (
+                [("*/*", 1), ("text/html", 1), ("image/*", 1)],
+                ["text/plain", "image/png"],
+                None,
+                "image/png",
+            ),
+            (
+                [("text/html", 1), ("text/html; level=1", 1)],
+                ["text/html;level=1"],
+                None,
+                "text/html;level=1",
+            ),
+        ],
+    )
+    def test_mime_accept(self, values, matches, default, expect):
+        accept = MIMEAccept(values)
+        match = accept.best_match(matches, default=default)
+        assert match == expect
 
-    def test_accept_wildcard_subtype(self):
-        accept = self.storage_class([("text/*", 1)])
-        assert accept.best_match(["text/html"], default=None) == "text/html"
-        assert accept.best_match(["image/png", "text/plain"]) == "text/plain"
-        assert accept.best_match(["image/png"], default=None) is None
 
-    def test_accept_wildcard_specificity(self):
-        accept = self.storage_class([("*/*", 1), ("text/html", 1)])
-        assert accept.best_match(["image/png", "text/html"]) == "text/html"
-        assert accept.best_match(["image/png", "text/plain"]) == "image/png"
-        accept = self.storage_class([("*/*", 1), ("text/html", 1), ("image/*", 1)])
-        assert accept.best_match(["image/png", "text/html"]) == "text/html"
-        assert accept.best_match(["text/plain", "image/png"]) == "image/png"
+class TestLanguageAccept(object):
+    @pytest.mark.parametrize(
+        ("values", "matches", "default", "expect"),
+        (
+            ([("en-us", 1)], ["en"], None, "en"),
+            ([("en", 1)], ["en_US"], None, "en_US"),
+            ([("en-GB", 1)], ["en-US"], None, None),
+            ([("de_AT", 1), ("de", 0.9)], ["en"], None, None),
+            ([("de_AT", 1), ("de", 0.9), ("en-US", 0.8)], ["de", "en"], None, "de"),
+            ([("de_AT", 0.9), ("en-US", 1)], ["en"], None, "en"),
+            ([("en-us", 1)], ["en-us"], None, "en-us"),
+            ([("en-us", 1)], ["en-us", "en"], None, "en-us"),
+            ([("en-GB", 1)], ["en-US", "en"], "en-US", "en"),
+            ([("de_AT", 1)], ["en-US", "en"], "en-US", "en-US"),
+            ([("aus-EN", 1)], ["aus"], None, "aus"),
+            ([("aus", 1)], ["aus-EN"], None, "aus-EN"),
+        ),
+    )
+    def test_best_match_fallback(self, values, matches, default, expect):
+        accept = LanguageAccept(values)
+        best = accept.best_match(matches, default=default)
+        assert best == expect
 
 
 class TestFileStorage(object):
@@ -1092,6 +1235,14 @@ class TestFileStorage(object):
         for name in ("fileno", "writable", "readable", "seekable"):
             assert hasattr(file_storage, name)
 
+    def test_save_to_pathlib_dst(self, tmp_path):
+        src = tmp_path / "src.txt"
+        src.write_text(u"test")
+        storage = self.storage_class(src.open("rb"))
+        dst = tmp_path / "dst.txt"
+        storage.save(dst)
+        assert dst.read_text() == "test"
+
 
 @pytest.mark.parametrize("ranges", ([(0, 1), (-5, None)], [(5, None)]))
 def test_range_to_header(ranges):
